#!/usr/bin/env python

"""
Generic Web framework interfaces.
The WebStack architecture consists of the following layers:

  * Framework - The underlying Web framework implementation.
  * Adapter   - Code operating under the particular framework which creates
                WebStack abstractions and issues them to the application.
  * Resources - Units of functionality operating within the hosted Web
                application.

Resources can act as both content producers within an application and as request
dispatchers to other resources; in the latter role, they may be referred to as
directors.
"""

class ContentType:

    "A container for content type information."

    def __init__(self, content_type, charset=None):
        self.content_type = content_type
        self.charset = charset

class Transaction:

    """
    A generic transaction interface containing framework-specific methods to be
    overridden.
    """

    def commit(self):

        """
        A special method, synchronising the transaction with framework-specific
        objects.
        """

        pass

    # Utility methods.

    def parse_content_type(self, content_type_field):

        """
        Determine the content type and charset from the supplied
        'content_type_field' string.
        """

        if content_type_field is None:
            return ContentType(None, "iso-8859-1")

        t = content_type_field.split("; charset=")
        if len(t) == 1:
            return ContentType(t[0], "iso-8859-1")
        else:
            return ContentType(t[0], t[1])

    def format_content_type(self, content_type):

        """
        Format the given 'content_type' object, producing a string suitable for
        the response header field.
        """

        if content_type.charset:
            field = "%s; charset=%s" % (content_type.content_type, content_type.charset)
        else:
            field = content_type.content_type

        # Make sure that only ASCII is used in the header.

        return field.encode("US-ASCII")

    def parse_content_preferences(self, accept_preference):

        """
        Returns the preferences as requested by the user agent. The preferences are
        returned as a list of codes in the same order as they appeared in the
        appropriate environment variable. In other words, the explicit weighting
        criteria are ignored.

        As the 'accept_preference' parameter, values for language and charset
        preferences are appropriate.
        """

        accept_defs = accept_preference.split(",")
        accept_prefs = []
        for accept_def in accept_defs:
            t = accept_def.split(";")
            if len(t) >= 1:
                accept_prefs.append(t[0].strip())
        return accept_prefs

    # Request-related methods.

    def get_request_stream(self):

        """
        A framework-specific method which returns the request stream for
        the transaction.
        """

        raise NotImplementedError, "get_request_stream"

    def get_request_method(self):

        """
        A framework-specific method which gets the request method.
        """

        raise NotImplementedError, "get_request_method"

    def get_headers(self):

        """
        A framework-specific method which returns the request headers.
        NOTE: Experimental, since framework support varies somewhat.
        """

        raise NotImplementedError, "get_headers"

    def get_content_type(self):

        """
        A framework-specific method which gets the content type specified on the
        request, along with the charset employed.
        """

        raise NotImplementedError, "get_content_type"

    def get_content_charsets(self):

        """
        Returns the character set preferences.
        """

        raise NotImplementedError, "get_content_charsets"

    def get_content_languages(self):

        """
        A framework-specific method which extracts language information from
        the transaction.
        """

        raise NotImplementedError, "get_content_languages"

    def get_path(self):

        """
        A framework-specific method which gets the entire path from the request.
        """

        raise NotImplementedError, "get_path"

    def get_path_info(self):

        """
        A framework-specific method which gets the "path info" (the part of the
        URL after the resource name handling the current request) from the
        request.
        """

        raise NotImplementedError, "get_path_info"

    def get_fields(self):

        """
        A framework-specific method which extracts the form fields from the
        transaction.
        """

        raise NotImplementedError, "get_fields"

    def get_agent_information(self):

        """
        A framework-specific method which extracts agent information from
        the transaction.
        """

        raise NotImplementedError, "get_agent_information"

    # Response-related methods.

    def get_response_stream(self):

        """
        A framework-specific method which returns the response stream for
        the transaction.
        """

        raise NotImplementedError, "get_response_stream"

    def get_response_code(self):

        """
        Get the response code associated with the transaction. If no response
        code is defined, None is returned.
        """

        raise NotImplementedError, "get_response_code"

    def set_response_code(self, response_code):

        """
        Set the 'response_code' using a numeric constant defined in the HTTP
        specification.
        """

        raise NotImplementedError, "set_response_code"

    def set_content_type(self, content_type):

        """
        A framework-specific method which sets the 'content_type' for the
        response.
        """

        raise NotImplementedError, "set_content_type"

class Resource:

    "A generic resource interface."

    def respond(self, trans):

        """
        An application-specific method which performs activities on the basis of
        the transaction object 'trans'.
        """

        raise NotImplementedError, "respond"

# vim: tabstop=4 expandtab shiftwidth=4
